/*  TI-89 Skip-Bo clone
    Copyright (C) 2009 Travis Evans
   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3.
   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
   
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* data.c
Functions dealing with manipulating game data (deck, cards, player
information, etc.) */

#include <stdlib.h>
#include <system.h>

#include "skipbo.h"
#include "data.h"
#include "graphics.h"

////////////////////////////////////////////////////////////////////////////////
void NewDeck (unsigned char *deck) {
	unsigned char position, card = CARD_1;

	// Cards 1 to 12
	for (position=0; position<=143; position++) {
		deck[position] = card;
		if ( (!((position+1) % 12)) && (position != 0) ) card++;
	}

	// Skip-bo cards
	for (position=144; position<DECK_SIZE; position++) {
		deck[position] = CARD_SKIPBO;
	}
}

////////////////////////////////////////////////////////////////////////////////
void ShuffleDeck (unsigned char *deck) {
	unsigned char size = SizeOfPile(deck, DECK_SIZE), newPos, pos, tmp;
	for (pos = 0; pos < size; pos++) {
		newPos = random(size);
		tmp = deck[newPos];
		deck[newPos] = deck[pos];
		deck[pos] = tmp;
	}
}

////////////////////////////////////////////////////////////////////////////////
unsigned char SizeOfPile (unsigned char *pile, unsigned char maxSize) {
	unsigned char position=0;
	while ( (pile[position] >= CARD_1 && pile[position] <= CARD_SKIPBO) &&
	position < maxSize)
		position++;
	return position;
}

////////////////////////////////////////////////////////////////////////////////
unsigned char GetCard(unsigned char pile, unsigned char playerNo, GAME *game,
PLAYER *player) {
	unsigned char pilePos = 0;
	if (pile <= BUILD_PILE_4) {
		pilePos = SizeOfPile(game->buildPile[pile], 12);
		if (pilePos>0) pilePos--;
		return game->buildPile[pile][pilePos];
	} else if (pile == STOCK_PILE) {
		pilePos = SizeOfPile(player[playerNo].stockPile, 30);
		if (pilePos>0) pilePos--;
		return player[playerNo].stockPile[pilePos];
	} else if (pile <= DISC_PILE_4) {
		pilePos = SizeOfPile(player[playerNo].discardPile[
			pile-DISC_PILE_1], DISCARD_PILE_SIZE);
		if (pilePos>0) pilePos--;
		return player[playerNo].discardPile[pile-DISC_PILE_1][
			pilePos];
	} else if (pile <= HAND_5) {
		pilePos = pile-HAND_1;
		return player[playerNo].hand[pilePos];
	} else if (pile == DRAW_PILE) {
		pilePos = SizeOfPile(game->drawPile, DECK_SIZE);
		if (pilePos>0) pilePos--;
		return game->drawPile[pilePos];
	}
	return CARD_OUTLINE;
}

////////////////////////////////////////////////////////////////////////////////
void PutCard(unsigned char pile, unsigned char card, unsigned char playerNo,
GAME *game, PLAYER *player) {
	unsigned char pilePos = 0;
	if (pile <= BUILD_PILE_4) {
		pilePos = SizeOfPile(game->buildPile[pile], 12);
		game->buildPile[pile][pilePos] = card;
		// Update base card equivalent
		if (pilePos == 0)
			game->buildPileBaseCard[pile] =
			(card != CARD_SKIPBO ? card : CARD_1);
	} else if (pile == STOCK_PILE) {
		pilePos = SizeOfPile(player[playerNo].stockPile, 30);
		player[playerNo].stockPile[pilePos] = card;
	} else if (pile <= DISC_PILE_4) {
		pilePos = SizeOfPile(
			player[playerNo].discardPile[pile-DISC_PILE_1],
			DISCARD_PILE_SIZE);
		player[playerNo].discardPile[pile-DISC_PILE_1][pilePos] = card;
	} else if (pile <= HAND_5) {
		pilePos = pile-HAND_1;
		player[playerNo].hand[pilePos] = card;
	} else if (pile == DRAW_PILE) {
		pilePos = SizeOfPile(game->drawPile, DECK_SIZE);
		game->drawPile[pilePos] = card;
	}
}

////////////////////////////////////////////////////////////////////////////////
void MoveCard(unsigned char oldPos, unsigned char newPos, unsigned char playerNo,
GAME *game, PLAYER *player) {
	unsigned char pilePos = 0, loop;

	// Copy the card to its new location
	PutCard(newPos, GetCard(oldPos, playerNo, game, player), playerNo, game,
		player);

	// Remove the original card
	if (oldPos <= BUILD_PILE_4) {
		pilePos = SizeOfPile(game->buildPile[oldPos], 12);
		if (pilePos>0)
			game->buildPile[oldPos][pilePos-1] = CARD_OUTLINE;
	} else if (oldPos == STOCK_PILE) {
		pilePos = SizeOfPile(player[playerNo].stockPile, 30);
		if (pilePos>0)
			player[playerNo].stockPile[pilePos-1] = CARD_OUTLINE;
	} else if (oldPos <= DISC_PILE_4) {
		pilePos = SizeOfPile(player[playerNo].discardPile[oldPos-DISC_PILE_1],
			DISCARD_PILE_SIZE);
		if (pilePos>0)
			player[playerNo].discardPile[oldPos-DISC_PILE_1][pilePos-1] =
				CARD_OUTLINE;
	} else if (oldPos <= HAND_5) {
		pilePos = oldPos-HAND_1;
			player[playerNo].hand[pilePos] = CARD_OUTLINE;
		/* The hand is a special case; slide all cards down to fill in empty
		   spaces
		*/
		for (loop = pilePos; loop <= 4; loop++) {
			player[playerNo].hand[loop] = player[playerNo].hand[loop+1];
		}
		player[playerNo].hand[4] = CARD_OUTLINE;
	} else if (oldPos == DRAW_PILE) {
		pilePos = SizeOfPile(game->drawPile, DECK_SIZE);
		if (pilePos>0)
			game->drawPile[pilePos-1] = CARD_OUTLINE;
	}
}

////////////////////////////////////////////////////////////////////////////////
unsigned char FillHand (unsigned char animation, unsigned char playerNo,
GAME *game, PLAYER *player) {
	unsigned char handSize = 0, success = TRUE;

	while (success && (handSize = SizeOfPile(player[playerNo].hand, 5)) < 5) {

		// Try to recover draw pile cards if we don't have any more
		if (SizeOfPile(game->drawPile, DECK_SIZE) == 0) {
			if (animation) Delay(80);
			if (game->numCmplBuildPiles) {
					// Completed building piles animation
					if (animation) CompletedBuildPileAnimation(game);
					ReplenishDrawPile(game);
					if (animation) {
					DrawGameStatus(game);
					ShuffleAnimation();
					DrawAllCards(game, player, playerNo);
					Delay(120);
					}
			} else {
					// Active building piles animation
					if (animation) BuildPileFlash(4, game);
					if (!(success = ReplenishDrawPile(game))) {
					// Game over
					game->status = ROUND_NOT_STARTED;
					} else {
					if (animation) {
						DrawBuildingPiles(game);
						ShuffleAnimation();
						DrawAllCards(game, player, playerNo);
						Delay(120);
					}
					}
			}

		} else {
			// We have a card to put in the player's hand
			if (animation) AnimateCard(DRAW_PILE, playerNo, game, player);
			MoveCard(DRAW_PILE, HAND_1 + handSize, playerNo, game, player);
			if (animation) {
					DrawDrawPile(game);
					AnimateCard(HAND_1 + handSize, playerNo, game, player);
					Delay(CARD_ANIMATE_SPEED);
			}
		}
	}
	DrawHand(player, playerNo);
	DrawGameStatus(game);
	return handSize == 5 ? TRUE : FALSE;
}

////////////////////////////////////////////////////////////////////////////////
unsigned char BuildEffectiveCard (unsigned char pileNo, GAME *game) {
	//unsigned char curCard, effCard = game->buildPileBaseCard[pileNo], pilePos = 0;
	unsigned char effCard = game->buildPileBaseCard[pileNo], pileSize = 0;
	pileSize = SizeOfPile(game->buildPile[pileNo], 12);
	if (pileSize != 0) effCard += pileSize - 1;
	return effCard;
}

////////////////////////////////////////////////////////////////////////////////
unsigned char CheckBuildPiles(unsigned char animation, unsigned char playerNo,
GAME *game, PLAYER *player) {
	unsigned char pile, card, areAnyCompleted = FALSE;
	for (pile=0; pile<4; pile++) {
		if (BuildEffectiveCard(pile, game) >= CARD_12) {
			// This pile is completed
			if (animation)	BuildPileFlash(pile, game);

			/* Copy completed pile to completed building piles stack, then
			   clear building pile */
			for (card=0; card<12; card++) {
					game->cmplBuildPile[game->numCmplBuildPiles][card] =
						game->buildPile[pile][card];
					game->buildPile[pile][card] = CARD_OUTLINE;
			}
			game->buildPileBaseCard[pile] = CARD_OUTLINE;
			game->numCmplBuildPiles++;
			areAnyCompleted = TRUE;
			}
		}
		if (animation && areAnyCompleted) {
			DrawGameStatus(game);
			DrawBuildingPiles(game);
		}
	return areAnyCompleted;
}

////////////////////////////////////////////////////////////////////////////////
unsigned char ReplenishDrawPile (GAME *game) {
	unsigned char success = 0, pileNo, srcPileIndex, srcPileSize;
	unsigned char drawPileIndex = SizeOfPile(game->drawPile, DECK_SIZE),

	/* First try to shuffle the completed building piles */
	if (game->numCmplBuildPiles > 0) {
		/* Copy cards from all completed building piles to draw pile */
		for (pileNo = 0; pileNo < game->numCmplBuildPiles; pileNo++) {
			srcPileSize = SizeOfPile(game->cmplBuildPile[pileNo], 12);
			for (srcPileIndex = 0; srcPileIndex < srcPileSize;
			srcPileIndex++) {
				game->drawPile[drawPileIndex] =
					game->cmplBuildPile[pileNo][srcPileIndex];
				drawPileIndex++;
			}
			/* Clear removed cards from completed building piles */
			game->cmplBuildPile[pileNo][0] = CARD_OUTLINE;
		}
		game->numCmplBuildPiles = 0;
		success = 1;

	/* If there aren't any completed building piles, try to use all possible
	   cards from the active building piles
	*/
	} else {
		for (pileNo = 0; pileNo < 4; pileNo++) {
			srcPileSize = SizeOfPile(game->buildPile[pileNo], 12);
			if (srcPileSize > 1) {
				// We're saved!
				success = 2;
				game->buildPileBaseCard[pileNo] =
					BuildEffectiveCard(pileNo, game);
				for (srcPileIndex = 0; srcPileIndex < srcPileSize-1;
				srcPileIndex++) {
					game->drawPile[drawPileIndex] =
						game->buildPile[pileNo][srcPileIndex];
					game->buildPile[pileNo][srcPileIndex] = CARD_OUTLINE;
					drawPileIndex++;
				}
				// Clear removed cards from building pile
				game->buildPile[pileNo][0] =
					game->buildPile[pileNo][srcPileSize-1];
				game->buildPile[pileNo][srcPileSize-1] = CARD_OUTLINE;
			}
		}
	}

	// Shuffle the draw pile if we got anything
	if (success) ShuffleDeck(game->drawPile);
	return success;
}
