/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* data.h
   Defines enumerations and data structures used for the game state */

#ifndef _DATA_H
#define _DATA_H

#define DECK_SIZE             162
#define DISCARD_PILE_SIZE     DECK_SIZE / 8 + 1

///// Enumerations /////////////////////////////////////////////////////////////

enum GAMETYPE       {STANDARD, QUICK};
enum DIFFICULTY     {EASY, NORMAL, HARD};
enum PLAYERS        {PLYR_A, PLYR_B, PLYR_C, PLYR_D};
enum STATUS         {GAME_NOT_STARTED, ROUND_NOT_STARTED, GAME_IN_PROGRESS};
enum CARDS          {CARD_OUTLINE, CARD_1, CARD_2, CARD_3, CARD_4, CARD_5,
				 CARD_6, CARD_7, CARD_8, CARD_9, CARD_10, CARD_11, CARD_12,
				 CARD_SKIPBO, CARD_BACK, CARD_HIGHLIGHTER, CARD_MASK,
				 CARD_X};

///// Data types ///////////////////////////////////////////////////////////////

/* Data used for a game */
typedef struct {
	// Game modes
	unsigned char type:1, numPlayers:2, difficulty:2, playerTurn:2;
		/* Type of game (short, regular), number of players, difficulty level
		   of computer players (easy, medium, hard), player currently playing
		   Note that number of players is one less than actual value (e.g.,
		   if numPlayers = 2, actual number of players is 3) */
	unsigned char status;                 // Game in progress, game finished, etc.
	// Piles
	unsigned char drawPile[DECK_SIZE];    // Draw pile (deck)
	unsigned char buildPileBaseCard[4];   // Values of bottommost building pile cards
	unsigned char buildPile[4][12];       // Building piles
	unsigned char numCmplBuildPiles;      // Number of completed building piles
	unsigned char cmplBuildPile[12][12];  // Storage for completed building piles
} GAME;

/* Data used for each player */
typedef struct {
	// Information about each player
	unsigned short score;                               // Player score
	unsigned char  hand[5];                             // Cards in hand
	unsigned char  discardPile[4][DISCARD_PILE_SIZE];   // Discard piles x 4
	unsigned char  stockPile[30];                       // Stock pile
} PLAYER;

///// Functions ////////////////////////////////////////////////////////////////

/* Create a new deck
   Adds twelve cards each of CARD_1 through CARD_12, then fills the rest of the
   deck (DECK_SIZE cards) with CARD_SKIPBO.
   deck: An array of cards */
void NewDeck (unsigned char *deck);

/* Shuffle an existing deck
   deck: An array of cards
*/
void ShuffleDeck (unsigned char *deck);

/* Return the number of cards in a deck or pile of cards
   Works by scanning entire pile until a card other than CARD_1 to CARD_12 or
   CARD_SKIPBO is reached.
   pile: Array of cards
   maxSize: Maximum size of pile array (required in case there is no terminator
     (non-1 to 12 or SKIP-BO) card found at the end of the array) */
unsigned char SizeOfPile (unsigned char *pile, unsigned char maxSize);

/* Return the topmost card of a deck or pile of cards
   pile: Pile to get the top card of; uses GPH_CURSOR_POSITIONS enum
   playerNo: Player number for pile
   game, player: Game and player structures */
unsigned char GetCard (unsigned char pile, unsigned char playerNo, GAME *game,
PLAYER *player);

/* Insert a card at the top of a pile
   pile: Pile to place card on (uses GPH_CURSOR_POSITIONS enum values)
   card: The card value to place (CARD_1, CARD_2, etc.; CARDS enum)
   playerNo: Player number for pile (PLYR_A, PLYR_B, etc.; PLAYERS enum)
   game, player: Game and player structures */
void PutCard (unsigned char pile, unsigned char card, unsigned char playerNo,
GAME *game, PLAYER *player);

/* Move a topmost card from one pile to the top of another
   The hand is treated a bit differently: a card is taken directly from the
   specified position of the hand. If a card is placed into the hand, it re-
   places whatever card was already there.
   Warning: Does not check for buffer overflows in the event of a pile receiving
   too many cards
   oldPos: Old pile (GPH_CURSOR_POSITIONS enum)
   newPos: New pile (GPH-CURSOR_POSITIONS enum)
   playerNo: Player number for pile (PLAYERS enum)
   game, player: Game and player structures */
void MoveCard (unsigned char oldPos, unsigned char newPos,
unsigned char playerNo, GAME *game, PLAYER *player);

/* Refill a player's hand
   Adds cards to the specified player's hand until the hand contains five cards
   animation: TRUE if animation effect should be displayed
   playerNo: Player to give cards to (PLAYERS enum)
   OUTPUT: FALSE if enough cards could not be recovered to fill the hand */
unsigned char FillHand (unsigned char animation, unsigned char playerNo,
GAME *game, PLAYER *player);

/* Find the effective card of a building pile card
   If the card at the top of the specified building pile is a number card,
   returns that card.  If it's a Skip-bo, this function determines what number
   card the Skip-bo replaces and returns that card.
   pileNo: Which building pile to check (1, 2, 3, or 4)
   game: Game structure */
unsigned char BuildEffectiveCard (unsigned char pileNo, GAME *game);

/* Check building piles for completion
   Checks all four building piles for completion (cards numbered up to 12) and
   moves any completed piles from the player area to the completed building
   pile storage pile.
   animation: TRUE if animation is to be displayed
   game, player: Game and player structures
   OUTPUT: TRUE if any piles were completed */
unsigned char CheckBuildPiles(unsigned char animation, unsigned char playerNo,
GAME *game, PLAYER *player);

/* Replenish draw pile
   Attempts to add more cards to the draw pile, first by trying to shuffle the
   completed building piles to form a new draw pile, and if there aren't any
   completed building piles, tries to shuffle all but the topmost cards of the
   building piles. Returns 1 if more cards were obtained by shuffling the
   completed building piles, 2 if more cards were obtained by removing cards
   from the active building piles, and 0 if there was no way to get any more
   cards (which ends the game) */
unsigned char ReplenishDrawPile(GAME *game);

#endif
