/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* gameplay.c
   Functions for actual gameplay and key input processing */

#include <estack.h>
#include <mem.h>
#include <stdio.h>
#include <stdlib.h>
#include <system.h>

#include "data.h"
#include "gameplay.h"
#include "graphics.h"
#include "input.h"

extern unsigned char cursorPosition, selectedCard;

///// File name strings ////////////////////////////////////////////////////////
static const char *saveFile = "skipbogm";
static const char *saveFileType = "SAV";

////////////////////////////////////////////////////////////////////////////////
void PlayGame (GAME *game, PLAYER *player) {
	unsigned short i, j, gameRunning = TRUE;

	/* Try to load last saved game */
	if (!LoadGame(game, player)) {
		/* Load failure; start new game */
		game->status = GAME_NOT_STARTED;
	} else {
		/* If load suceeded, initialize game screen */
		DrawWholeScreen(game, player, PLYR_A);
	}

	/* Main game loop */
	while (gameRunning) {

		switch (game->status) {
			case GAME_NOT_STARTED:        // Get game options
				/* Iniitalize game structure */
				memset(game, 0, sizeof(GAME));
				memset(player, 0, sizeof(PLAYER)*4);
				game->numPlayers = 1;
				NewDeck(game->drawPile);
				/* Initialize game screen */
				DrawWholeScreen(game, player, game->playerTurn);

				/* Get game options */
				gameRunning = GetGameOptions(game);
				if (gameRunning) game->status = ROUND_NOT_STARTED;
			break;
			case ROUND_NOT_STARTED:       // Initialize game
				/* Shuffle cards */
				DrawWholeScreen(game, player, game->playerTurn);
				Delay(100);
				ShuffleDeck(game->drawPile);
				ShuffleAnimation();

				///////////// Test code; remove later! ///////////////
				//for (i=0; i<30; i++) {
				//     MoveCard(DRAW_PILE, STOCK_PILE, PLYR_A, game, player);
				//     MoveCard(DRAW_PILE, STOCK_PILE, PLYR_B, game, player);
				//     MoveCard(DRAW_PILE, STOCK_PILE, PLYR_C, game, player);
				//     MoveCard(DRAW_PILE, STOCK_PILE, PLYR_D, game, player);
				//}
				FillHand(FALSE, PLYR_A, game, player);
				///////////////////////////////////////////////////////

				/* Give stock pile cards to each player */
				for (i = PLYR_A; i <= game->numPlayers; i++) {

					/* Display stock pile deal animation */
					DrawWholeScreen(game, player, i);
					DealStockPileAnimation(
						(game->type == STANDARD) ? 30 : 10);
					Delay(100);

					/* Add actual cards to stock pile */
					for (j = 0; j < ((game->type == STANDARD) ? 30 : 10);
					j++) {
						MoveCard(DRAW_PILE, STOCK_PILE, i, game, player);
					}
				}

				/* Randomly determine beginning player */
				//game->playerTurn = random(game->numPlayers);
				game->playerTurn = PLYR_A;
				DrawAllCards(game, player, PLYR_A);
				DrawPlayers(player, PLYR_A);

				/* Initialize cursor and selected card */
				cursorPosition = 0;
				selectedCard = NO_CARD_SELECTED;

				/* Begin game */
				game->status = GAME_IN_PROGRESS;

			/* No break needed; run following GAME_IN_PROGRESS routine next */
			case GAME_IN_PROGRESS:
				/* Actual game play */
				UpdatePileIndicator(cursorPosition);
				gameRunning =
					ProcessInGameKey(
						GetKeyEvent(TRUE, PLYR_A, game, player),
						game, player);
		}
	}

	SaveGame(game, player);
}

////////////////////////////////////////////////////////////////////////////////
unsigned char SaveGame(GAME *game, PLAYER *player) {
	FILE *sf = NULL;
	unsigned char *structPtr = NULL; unsigned short counter;
	unsigned short checksum;

	sf = fopen(saveFile, "wb");
	if (sf == NULL) return FALSE;

	/* Write save data */
	fputc(cursorPosition, sf);
	fputc(selectedCard, sf);
	fwrite(game, sizeof(GAME), 1, sf);
	fwrite(player, sizeof(PLAYER), 4, sf);

	checksum = cursorPosition + selectedCard;
	structPtr = (unsigned char *)game;
	for (counter = 0; counter < sizeof(GAME); counter++) {
		checksum += structPtr[counter];
	}
	structPtr = (unsigned char *)player;
	for (counter = 0; counter < sizeof(PLAYER)*4; counter++) {
		checksum += structPtr[counter];
	}
	fwrite(&checksum, sizeof(checksum), 1, sf);

	/* Ending file tag */
	fputc(0, sf);
	fputs(saveFileType, sf);
	fputc(0, sf);
	fputc(OTH_TAG, sf);

	/* Return */
	fclose(sf);
	return !ferror(sf);
}

////////////////////////////////////////////////////////////////////////////////
unsigned char LoadGame(GAME *game, PLAYER *player) {
	FILE *sf = NULL; unsigned char itemsRead = 0;
	unsigned short fileChecksum, actChecksum;
	unsigned char *structPtr = NULL; unsigned short counter;

	sf = fopen(saveFile, "rb");
	if (sf == NULL) return FALSE;

	/* Read save data */
	cursorPosition = fgetc(sf);
	selectedCard = fgetc(sf);
	itemsRead += fread(game, sizeof(GAME), 1, sf);
	itemsRead += fread(player, sizeof(PLAYER), 4, sf);

	/* Check for proper checksum */
	itemsRead += fread(&fileChecksum, sizeof(fileChecksum), 1, sf);
	actChecksum = cursorPosition + selectedCard;
	structPtr = (unsigned char *)game;
	for (counter = 0; counter < sizeof(GAME); counter++) {
		actChecksum += structPtr[counter];
	}
	structPtr = (unsigned char *)player;
	for (counter = 0; counter < sizeof(PLAYER)*4; counter++) {
		actChecksum += structPtr[counter];
	}

	/* Return */
	fclose(sf);
	return ((itemsRead = 6) && actChecksum == fileChecksum ? TRUE : FALSE);
}

////////////////////////////////////////////////////////////////////////////////
unsigned char ProcessInGameKey (unsigned char key, GAME *game, PLAYER *player) {
	unsigned char lastHandCard = HAND_5;

	/* Erase cursor */
	CursorOff();

	/* Get last hand card (to prevent cursor from appearing in the hand at a
	   position where no card exists) */
	lastHandCard = HAND_1 + SizeOfPile(player[PLYR_A].hand, 5) - 1;

	/* Process key */
	switch (key) {

		/* Cursor movement keys */
		case KE_RIGHT:
			if (cursorPosition <= STOCK_PILE) {
				/* Cursor is on top row */
				cursorPosition =
					cursorPosition < STOCK_PILE ?
					cursorPosition+1 : BUILD_PILE_1;
			} else {
				/* Cursor is on bottom row */
				cursorPosition =
					cursorPosition < lastHandCard ?
					cursorPosition+1 : DISC_PILE_1;
			}
			break;
		case KE_LEFT:
			if (cursorPosition <= STOCK_PILE) {
				/* Cursor is on top row */
				cursorPosition =
					cursorPosition > BUILD_PILE_1 ?
					cursorPosition-1 : STOCK_PILE;
			} else {
				/* Cursor is on bottom row */
				cursorPosition =
					cursorPosition > DISC_PILE_1 ?
					cursorPosition-1 : lastHandCard;
			}
			break;
		case KE_UP: case KE_DOWN:
			if (cursorPosition <= STOCK_PILE) {
				/* Cursor is on top row */
				cursorPosition += 5;
			} else if (cursorPosition <= HAND_1) {
				/* Cursor is on bottom row on discard piles or first
					card of hand */
				cursorPosition -= 5;
			} else {
				/* Cursor is on bottom row and on hand cards 2 to 5 */
				cursorPosition = STOCK_PILE;
			}
			break;

		/* Select and cancel keys */
		case KE_SELECT:
			UpdatePileIndicator(NO_CARD_SELECTED);
			if (selectedCard == NO_CARD_SELECTED) {
				/* First card being selected */
				if (GetCard(cursorPosition, PLYR_A, game, player) !=
				CARD_OUTLINE && cursorPosition > BUILD_PILE_4) {
					selectedCard = cursorPosition;
					UpdateSelection(game, player, PLYR_A);
				}
			} else if (selectedCard == cursorPosition) {
				/* Card selection cancelled (user tried to move a card onto
				   itself */
				selectedCard = NO_CARD_SELECTED;
				UpdateSelection(game, player, PLYR_A);
			} else {
				/* First card already selected; destination chosen */
				if (IsLegalMove(selectedCard, cursorPosition, PLYR_A, game,
				player)) {
					MoveCard(selectedCard, cursorPosition, PLYR_A, game,
						player);
					selectedCard = NO_CARD_SELECTED;
					DrawAllCards(game, player, PLYR_A);
					AnimateCard(cursorPosition, PLYR_A, game, player);
					/* Temporary debugging code */
					if ((cursorPosition >= DISC_PILE_1 &&
					cursorPosition <= DISC_PILE_4) ||
					SizeOfPile(player[PLYR_A].hand, 5) == 0) {
						FillHand(TRUE, PLYR_A, game, player);
					}
					if (CheckBuildPiles(TRUE, PLYR_A, game, player)) {
						DrawAllCards(game, player, PLYR_A);
					}
				} else {
					/* Don't allow illegal card movements; select as new
					   card instead */
					if (GetCard(cursorPosition, PLYR_A, game, player) !=
					CARD_OUTLINE && cursorPosition > BUILD_PILE_4) {
						selectedCard = cursorPosition;
						DrawAllCards(game, player, PLYR_A);
					}
				}
				UpdateSelection(game, player, PLYR_A);

			}
			break;
		case KE_CANCEL:
			selectedCard = NO_CARD_SELECTED;
			UpdateSelection(game, player, PLYR_A);
			break;

		/* New Game key */
		case KE_NEW_GAME:
			PrintNewGameConfirmation();
			Delay(50);
			key = KE_NO_KEY;
			while (key != KE_SELECT && key != KE_QUIT) {
				key = GetKeyEvent(FALSE, 0, NULL, NULL);
			}
			/* The following strange line was discovered after a long hiatus
			   from working on this project, right here at this location in
			   this source file, without being commented out or anything.  The
			   only explanation I can come up for it is that I had copied it
			   from a forum and accidentally pasted it into this source without
			   realizing it. */
			// I DONT GIVE A ****!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif mad.gif

			if (key == KE_SELECT) {
				game->status = GAME_NOT_STARTED;
				return TRUE;
			}
			DrawWholeScreen(game, player, game->playerTurn);
			break;

		/* Off key */
		case KE_OFF:
			off();
			break;
		/* Quit key */
		case KE_QUIT:
			return FALSE;
	}

	/* Redraw cursor */
	CursorOn(game, player, PLYR_A);
	UpdatePileIndicator(cursorPosition);
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////
unsigned char GetGameOptions (GAME *game) {
	unsigned char key = KE_NO_KEY;

	/* Display options instructions */
	PrintGameOptionsHelp();

	/* Get keypresses and act on them */
	while (key != KE_SELECT && key != KE_QUIT) {
		key = GetKeyEvent(FALSE, 0, NULL, NULL);

		/* Each key cycles through its corresponding settings.  When a setting
		   reaches its maximum value, it resets back to its lowest value */
		switch (key) {
			case KE_LEFT:
				game->type++;
			break;
			case KE_RIGHT:
				game->difficulty++;
				if (game->difficulty > HARD) game->difficulty = EASY;
			break;
			case KE_UP:
				game->numPlayers++;
				/* Number of players is actually 1 + game->numPlayers
				   Only values 2, 3, and 4 are allowed */
				if (game->numPlayers <= 0) game->numPlayers = 1;
		}

		DrawGameStatus(game);
	}

	/* Done */
	return (key == KE_SELECT ? TRUE : FALSE);
}

////////////////////////////////////////////////////////////////////////////////
unsigned char IsLegalMove (unsigned char oldPos, unsigned char newPos,
unsigned char playerNo, GAME *game, PLAYER *player) {
	unsigned char srcCard, dstCard;

	/* Moves TO anything but the discard pile or building piles are never
	   legal (adding cards to stock pile or hand should be done
	   automatically by the game, without checking for legality) */
	if (newPos == STOCK_PILE || newPos > DISC_PILE_4) return FALSE;

	/* Moves FROM anything but the stock pile, hand, or discard piles are
	   also not allowed */
	if (oldPos < STOCK_PILE || oldPos > HAND_5) return FALSE;

	/* Moves from stock pile to hand or from discard pile to discard pile are
	   not allowed */
	if ((oldPos == STOCK_PILE && newPos >= DISC_PILE_1 && newPos <= DISC_PILE_4)
		|| (oldPos >= DISC_PILE_1 && oldPos <= DISC_PILE_4 &&
		newPos >= DISC_PILE_1 && newPos <= DISC_PILE_4)) return FALSE;

	/* Moving to discard pile is technically never illegal, but this code
	   protects against buffer overflows of the discard pile in the highly
	   unlikely event that it ever gets too big */
	if (newPos >= DISC_PILE_1 && newPos <= DISC_PILE_4) {
		if (SizeOfPile(player[playerNo].discardPile[newPos-DISC_PILE_1],
			DISCARD_PILE_SIZE) >= DISCARD_PILE_SIZE) return FALSE;
	}

	/* Moving to building pile is only legal if card is CARD_SKIPBO or if it
	   is a number card that is one higher than the top card of the building
	   pile */
	srcCard = GetCard(oldPos, playerNo, game, player);
	dstCard = BuildEffectiveCard(newPos, game);
	if (newPos <= BUILD_PILE_4) {
		if ( (dstCard + 1 != srcCard) && (srcCard != CARD_SKIPBO) )
			return FALSE;
	}

	/* If we made it this far, the move must be legal */
	return TRUE;
}
