/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* graphics.h
   Defines the sprites and graphics used in the game */

#ifndef _GRAPHICS_H
#define _GRAPHICS_H

#define CARD_ANIMATE_SPEED       30

#include "data.h"

///// Game Screen //////////////////////////////////////////////////////////////

enum GPH_SCREEN_ITEMS {SCRN_ALL, SCRN_FRAMEWORK, SCRN_PLAYERS, SCRN_ALL_CARDS,
	SCRN_GAME_STATUS, SCRN_DRAW_PILE, SCRN_BUILD_PILES, SCRN_STOCK_PILE,
	SCRN_DISCARD_PILES, SCRN_HAND, SCRN_CURSOR_OFF, SCRN_CURSOR_ON,
	SCRN_CURSOR_TOGGLE, SCRN_CARD_HIGHLIGHT};

///// Card cursor positions ////////////////////////////////////////////////////

enum GPH_CURSOR_POSITIONS {BUILD_PILE_1, BUILD_PILE_2, BUILD_PILE_3,
	BUILD_PILE_4, STOCK_PILE, DISC_PILE_1, DISC_PILE_2, DISC_PILE_3,
	DISC_PILE_4, HAND_1, HAND_2, HAND_3, HAND_4, HAND_5, DRAW_PILE,
	NO_CARD_SELECTED};

///// Functions ////////////////////////////////////////////////////////////////

/* Draws main boxes and help text of game screen */
void DrawFramework(void);

/* Draws player score information */
void DrawPlayers(PLAYER *player, unsigned char playerNo);

/* Draws game modes and status */
void DrawGameStatus(GAME *game);

/* Draws number of cards in draw pile */
void DrawDrawPile(GAME *game);

/* Draws a pile of cards so that it looks like a stack of cards (extra
   rows of black pixels above pile depending on how many cards are in
   the pile)
   x, y: Coordinates to place the pile on screen
   pile: Array of cards to draw
   pileSize: Number of cards in array */
void DrawCompressedPile(unsigned char x, unsigned char y, unsigned char *pile,
unsigned char pileSize);

/* Draws the building piles */
void DrawBuildingPiles(GAME *game);

/* Draws the stock pile
   playerNo: Which player whose stock pile is to be drawn */
void DrawStockPile(PLAYER *player, unsigned char playerNo);

/* Draws the discard piles
   playerNo: The player whose discard pile is to be drawn  */
void DrawDiscardPiles(PLAYER *player, unsigned char playerNo);

/* Draws a player's hand
   playerNo: The player whose hand is to be drawn */
void DrawHand(PLAYER *player, unsigned char playerNo);

/* Draws all cards
   Calls DrawDrawPile, DrawBuildingPiles, DrawStockPile, DrawDiscardPiles,
   and DrawHand */
void DrawAllCards(GAME *game, PLAYER *player, unsigned char playerNo);

/* Draws entire screen, from framework to game status to cards */
void DrawWholeScreen(GAME *game, PLAYER *player, unsigned char playerNo);

/* Erases the portion of the screen taken up by the playing cards */
void EraseCardArea(void);

/* Erases the portion of the screen taken up by the playing cards
   and displays the help text for setting game options before a
   game starts */
void PrintGameOptionsHelp(void);

/* Erases the portion of the screen taken up by the playing cards
   and displays the confirmation message for starting a new game */
void PrintNewGameConfirmation(void);

/* Erases the card cursor */
void CursorOff(void);

/* Draws the card cursor */
void CursorOn(GAME *game, PLAYER *player, unsigned char playerNo);

/* Toggles the card cursor */
void CursorToggle(GAME *game, PLAYER *player, unsigned char playerNo);

/* Redraws a pile of cards affected by the currently selected pile
   Causes the card selection (a selected card is drawn in inverted pixels)
   to be updated */
void UpdateSelection(GAME *game, PLAYER *player, unsigned char playerNo);

/* Briefly displays the top card of the given pile in inverted pixels
   pile: The pile (GPH_CURSOR_POSITIONS enum) to animate
   playerNo: The player of the playing field currently shown on the screen */
void AnimateCard(unsigned char pile, unsigned char playerNo, GAME *game,
PLAYER *player);

/* Displays an indicator below the hand to indicate which pile the cursor is
   currently on
   pile: The pile (GPH_CURSOR_POSITIONS enum) the cursor is on. The indicator
	    can be erased by using NO_CARD_SELECTED.  DRAW_PILE is not supported.
   */
void UpdatePileIndicator(unsigned char pile);

/* Displays the flashing "shuffle" animation
   DrawAllCards() should be called afterwards to restore the screen */
void ShuffleAnimation(void);

/* Flashes the completed building piles count indicator a few times */
void CompletedBuildPileAnimation(GAME *game);

/* Flashes a building pile, or all building piles simultaneously
   pileNo: Which pile to flash (0-3).  4 will cause all piles to flash
   playerNo: The player whose playing field is currently on the screen */
void BuildPileFlash(unsigned char pileNo, GAME *game);

/* Displays the animation for dealing the stock pile to a player.  This does
   not actually deal the stock pile.
   numCards: Number of cards to show being dealt */
void DealStockPileAnimation(unsigned char numCards);

/* Temporarily pauses program execution
   length: The number of AUTO_INT_5 interrupts to wait before continuing */
void Delay(unsigned long length);

#endif
