/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <kbd.h>
#include <graph.h>
#include <intr.h>
#include <stdlib.h>
#include <sprites.h>
#include <system.h>

#include "skipbo.h"
#include "data.h"
#include "gameplay.h"
#include "graphics.h"

/* ==== TODO ====
   (Stuff that needs to be done first on top)
   RESUME:
    - Rearrange code to reflect final form better
*/

///// Text string constants ////////////////////////////////////////////////////
/* Error messages */
static const char *str_memErr1 = "Insufficient RAM to start";
static const char *str_memErr2 = "game. Please free some RAM";
static const char *str_memErr3 = "and try again.";
static const char *str_memErr4 = "Press any key to quit\xA0";

///// Custom AUTO_INT_5 interrupt handler //////////////////////////////////////
/* This is used to increment the system timer while the program is running.
   Normally this is done automatically by the system AUTO_INT_5, but since the
   system AUTO_INT_5 supposedly interferes with low-level keyboard input (which
   we do in this program), we disable it (along with AUTO_INT_1, which also
   interferes) and replace it with our own. */
DEFINE_INT_HANDLER(MyAutoInt5) {
	FiftyMsecTick++;
}

///// Main function ////////////////////////////////////////////////////////////
void _main(void) {
	INT_HANDLER saveAutoInt1 = GetIntVec(AUTO_INT_1);
	INT_HANDLER saveAutoInt5 = GetIntVec(AUTO_INT_5);
	unsigned char savePRGStart = PRG_getStart();
	GAME *game = NULL; PLAYER *player = NULL;

	/* === Initialization === */

	/* Initialize data structures */
	game = (GAME *)malloc(sizeof(GAME));
	player = (PLAYER *)malloc(sizeof(PLAYER)*4);
	if (!(game && player)) {
		ClrScr();
		DrawStr(0,28,str_memErr1,A_NORMAL);
		DrawStr(0,36,str_memErr2,A_NORMAL);
		DrawStr(0,44,str_memErr3,A_NORMAL);
		DrawStr(0,60,str_memErr4,A_NORMAL);
		ngetchx();
		if (game) free(game);
		if (player) free(player);
		return;
	}

	/* Randomization */
	srand(FiftyMsecTick);

	/* Interrupt initialization
	   Replace AUTO_INT_1 with a dummy handler (effectively disabling it) and
	   replace AUTO_INT_5 with our own simple one that only increments the
	   system timer and nothing else
	*/
	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
	SetIntVec(AUTO_INT_5, MyAutoInt5);
	/* Reprogram AUTO_INT_5 to trigger at a more suitable rate for this program.
	   This means that FiftyMsecTick actually increments several times more
	   often than once every 50 milliseconds while this program runs.  The
	   faster rate is needed for higher timing precision, especially in the
	   keyboard input routine where key "debouncing" is implemented.
	*/
	PRG_setStart(248);

	/* === Run the actual game === */
	PlayGame(game, player);

	/* === Clean-up === */
	/* Restore interrupt vectors */
	PRG_setStart(savePRGStart);
	SetIntVec(AUTO_INT_1, saveAutoInt1);
	SetIntVec(AUTO_INT_5, saveAutoInt5);

	/* Make sure the key pressed to exit the program doesn't also get passed to
	   AMS or to the calling program
	*/
	Delay(10);
	GKeyFlush();

	/* Deallocate memory that was used */
	free(game); free(player);
}
